import MDAnalysis as mda
from MDAnalysis.analysis.base import AnalysisFromFunction
from lipyphilic.leaflets.assign_leaflets import AssignLeaflets
from lipyphilic.analysis.memb_thickness import MembThickness
from lipyphilic.analysis.area_per_lipid import AreaPerLipid
import numpy as np
import sys

# time
start_time = 200              #in ns
end_time = 500                #in ns

def get_frames(u):

    dt = u.trajectory.dt/1000    #in ns
    start_frame = int(start_time/dt)
    end_frame = int(end_time/dt+1)
    no_frames = end_frame - start_frame

    return(start_frame, end_frame, no_frames)

def find_leaflets(u):

    print('Find leaflets')
    leaflets = AssignLeaflets(
               universe=u,
               lipid_sel="name PO4 PO41 PO42",
               )

    leaflets.run(
            start=start_frame,
            stop=end_frame,
            step=None,
            verbose=True
            )

    return(leaflets)

def memb_thick(u, leaflets):

    print('Calculating membrane thickness')
    memb_thickness = MembThickness(
                     universe=u,
                     leaflets=leaflets.leaflets,
                     lipid_sel="name PO4 PO41 PO42"
                     )

    memb_thickness.run(
            start=start_frame,
            stop=end_frame,
            step=None,
            verbose=True
            )

    return(memb_thickness)

def area_per_lipid(u, leaflets):

    print('Calculating area per lipid')
    areas = AreaPerLipid(
            universe=u,
            lipid_sel="name PO4 PO41 PO42",
            leaflets=leaflets.leaflets
            )

    areas.run(
           start=start_frame,
           stop=end_frame,
           step=None,
           verbose=True
           )

    return(areas)

#######################################################################################

lipids = ['POPC', 'CDL0']

path = sys.argv[1]

GRO = path + 'md.gro'
XTC = path + 'md.xtc'
u = mda.Universe(GRO, XTC)
start_frame, end_frame, no_frames = get_frames(u)

leaflets = find_leaflets(u)
upper_leaflet = []
lower_leaflet = []

head_groups = u.select_atoms("name PO4 GL0")
for frame in range(no_frames):
    lower_leaflet.append(head_groups[leaflets.leaflets[:,frame] == -1])
    upper_leaflet.append(head_groups[leaflets.leaflets[:,frame] == 1])

memb_thickness = (memb_thick(u, leaflets))
print('Average membrane thicknes: {:.2f} +/- {:.2f} nm'.format(np.mean(memb_thickness.memb_thickness)/10, np.std(memb_thickness.memb_thickness)/10))

areas = area_per_lipid(u, leaflets)
for lidx, lipid in enumerate(lipids):
    selection = u.select_atoms("not resname W ION")
    filter = (selection.residues.resnames == lipid)
    print('Average area per lipid for {}: {:.2f} +/- {:.2f} nm^2'.format(lipid, np.nanmean(areas.areas[filter,:]/100), np.nanstd(areas.areas[filter,:]/100)))
