#!/bin/bash

path_itp = "path_to_itp_folder"
path_ts2cg = "path_to_TS2CG_folder"
path_mdp = "path_to_mdp_folder"

#################################################################
# PCG parameters
#################################################################

#Curved system
x=15
y=10
z=12

#curvature amplitude, frequency and shift
A=2
omega=1
pshift=-90

#low wall bead density
wall_density=0.7

#point density for lipid placement
density=2

#################################################################
#Create a curved bilayer
#################################################################

function make_bilayer_curved {

mkdir $system-CURVED
cd $system-CURVED

cp $path_ts2cg/input_template.str input-$system.str

sed -i "s/LIP1  lip1_up  lip1_down  lip1_apl/"$lip1" "$lip1_rat"  "$lip1_rat"  "$apl1"/" input-$system.str
sed -i "s/LIP2  lip2_up  lip2_down  lip2_apl/"$lip2" "$lip2_rat"  "$lip2_rat"  "$apl2"/" input-$system.str
sed -i "s/Box  X  Y  Z/Box "$x" "$y" "$z"/"  input-$system.str
sed -i "s/Density  dens_up  dens_down/Density "$density" "$density"/"  input-$system.str
sed -i "s/WallDensity  wall_up  wall_down/WallDensity "$wall_density" "$wall_density"/"  input-$system.str
sed -i "s/Thickness D/Thickness "$thickness"/"  input-$system.str
sed -i "s/Mode  amp  per  shift/Mode "$A" "$omega"  "$pshift"/"  input-$system.str

mkdir 1_PREP
cd 1_PREP

$path_ts2cg/PCG  -str ../input-$system.str  -Bondlength  0.25  -LLIB  $path_ts2cg/Martini3.LIB -function analytical_shape -WallBName  WL  -WallH  0.1  -Wall

rm  topol.top
cat $path_itp/include.txt output.top >> topol.top

sed -i "s|path_to_itp|$path_itp|g" topol.top

cd ..
}

##############################################################
#energy minimize in vacuum
##############################################################

function EM_vac {

mkdir 2_EM_vac
cd 2_EM_vac

gmx grompp -f $path_mdp/em_vacc_sc.mdp -c ../1_PREP/output.gro -p ../1_PREP/topol.top -r ../1_PREP/output.gro -o em_vacc.tpr -maxwarn 1
gmx mdrun -deffnm em_vacc -v

echo -e "keep 0 \n r $lip1 | r $lip2 \n name 1 Bilayer \n case \n r wall \n q" | gmx make_ndx -f em_vacc.gro

gmx grompp -f $path_mdp/nvt_vacc.mdp -c em_vacc.gro -r em_vacc.gro -p ../1_PREP/topol.top -o nvt_vacc.tpr -n index.ndx
gmx mdrun -v -deffnm nvt_vacc -nb gpu -bonded gpu

rm \#*
cd ..
}

##################################################################
#Solvate & addition of ions
##################################################################

function solvate {
mkdir 3_SOL
cd 3_SOL

$path_ts2cg/SOL -in ../2_EM_vac/nvt_vacc.gro -Rcutoff 0.32 -tem $path_ts2cg/water.gro -o sol.gro

calculate_ions
rm info.txt

$path_ts2cg/SOL -in ../2_EM_vac/nvt_vacc.gro -Rcutoff 0.32 -tem $path_ts2cg/water.gro -o sol.gro -ion $ions_p $ions_n #adjust

rm topol.top
cp ../1_PREP/topol.top .
cat info.txt >> topol.top

gmx grompp -f $path_mdp/em.mdp -c sol.gro -p topol.top -r sol.gro -o em.tpr -maxwarn 1
gmx mdrun -deffnm em -v

rm \#*
cd ..
}
#######################################################################
#equilibration
#######################################################################

function nvt {
mkdir 4_EQ
cd 4_EQ

echo -e "keep 0 \n r ${lip1:0:4} | r ${lip2:0:4} \n name 1 Bilayer \n r W | a Na | a Cl \n name 2 W_ION \n case \n r wall \n q" | gmx make_ndx -f ../3_SOL/sol.gro

gmx grompp -f $path_mdp/nvt.mdp -c ../3_SOL/em.gro -r ../3_SOL/em.gro -p ../3_SOL/topol.top -o nvt.tpr -n index.ndx -maxwarn 1
gmx mdrun -v -deffnm nvt -nb gpu -bonded gpu -pin on
}

function npt_1 {
gmx grompp -f $path_mdp/npt_1.mdp -c nvt.gro -r nvt.gro -p ../3_SOL/topol.top -o npt_1.tpr -n index.ndx -maxwarn 1
gmx mdrun -v -deffnm npt_1 -nb gpu -bonded gpu -pin on
}

function npt_2 {
gmx grompp -f $path_mdp/npt_2.mdp -c npt_1.gro -r npt_1.gro -p ../3_SOL/topol.top -o npt_2.tpr -n index.ndx -maxwarn 1
gmx mdrun -v -deffnm npt_2 -nb gpu -bonded gpu -pin on

rm \#*
cd ..

mkdir 5_PROD
cd 5_PROD
}

#######################################################################
#Flat production run
#######################################################################

function RUN_curved {
gmx grompp -f $path_mdp/md.mdp -c ../4_EQ/npt_2.gro -r ../4_EQ/npt_2.gro -p ../3_SOL/topol.top -n ../4_EQ/index.ndx -o md.tpr -maxwarn 1
gmx mdrun -v -deffnm md -pin on -nsteps 2500000 -nb gpu -bonded gpu

rm \#*
cd ../../

}

#######################################################################
#Calculate charged of systems
#######################################################################

function calculate_ions {

lip1_count="$(awk /$lip1/'{sum += $NF} END {print sum}' ../1_PREP/output.top)"
lip2_count="$(awk /$lip2/'{sum += $NF} END {print sum}' ../1_PREP/output.top)"
water_count="$(awk /W/'{water = $NF} END {print water}' ../3_SOL/info.txt)"
total_charge=$(($lip1_count * $charge_1 + $lip2_count * $charge_2))

ions=$(bc <<< " $water_count * 4 * $ion_conc / 55.08")

if [[ $total_charge -ge 0 ]]
	then ions_p=$ions
	else ions_p=$(($total_charge * -1 + $ions))
fi
if [[ $total_charge -le 0 ]]
	then ions_n=$ions
	else ions_n=$(($total_charge+$ions))
fi
}

#######################################################################
#Calculate box resizing
#######################################################################

function check_box {

        cd $system-CURVED/5_PROD
        echo 11 12 13 0 | gmx energy -f md.edr -o box.xvg -b 30000
        avg_x=($(grep -v '^[@#]' box.xvg | awk 'BEGIN{x=0; count=0}{ x=x+$2 } {(( count++ ))} END{printf "%.5f\n", (x/count)}'))
        printf '%6.3f %s %6.3f %s %8.4f \n' $apl1_updated $lip1 $apl2_updated $lip2 $avg_x >> ../find_APL.dat
        lower_limit=$(echo $x*0.999 | bc)
        upper_limit=$(echo $x*1.001 | bc)
        scale_factor=$(echo $avg_x/$x | bc -l)
        if (( $(echo "$lower_limit < $avg_x" | bc) && $(echo "$upper_limit > $avg_x" | bc) ))
                then
                box_eq="true"
                else
                if (( $(echo "$x < $avg_x" | bc) ))
                        then
                        if (( $(echo "$avg_x - $x >= 0.1" | bc) ))
                                then
                                apl1_updated=$(echo $apl1_updated*$scale_factor | bc -l)
                                apl2_updated=$(echo $apl2_updated*$scale_factor | bc -l)
                                else
                                apl1_updated=$(echo $apl1_updated*1.001 | bc -l)
                                apl2_updated=$(echo $apl2_updated*1.001 | bc -l)
                        fi
                        else
                        if (( $(echo "$x - $avg_x >= 0.1" | bc) ))
                                then
                                apl1_updated=$(echo $apl1_updated*$scale_factor | bc -l)
                                apl2_updated=$(echo $apl2_updated*$scale_factor | bc -l)
                                else
                                apl1_updated=$(echo $apl1_updated*0.999 | bc -l)
                                apl2_updated=$(echo $apl2_updated*0.999 | bc -l)
                        fi
                fi

        fi
        cd ../..

	apl1=$apl1_updated
	apl2=$apl2_updated
}

#######################################################################
#Create & Run a curved system
#######################################################################

function curved_run {
        apl1_updated=$apl1
        apl2_updated=$apl2
        box_eq="false"
        while [ $box_eq !=  "true" ]
        do
        	make_bilayer_curved
        	EM_vac
        	solvate
        	nvt
        	npt_1
        	npt_2
        	RUN_curved
        	check_box
        done
}

#######################################################################
#Do everything
#######################################################################

function full_run {
      system=$1-$2-$7-$8
      lip1=$1
      lip2=$2
      apl1=$3
      apl2=$4
      charge_1=$5
      charge_2=$6
      lip1_rat=$7
      lip2_rat=$8
      thickness=$9
      ion_conc=${10}
      curved_run
}

#######################################################################
#Start a full run input Lipids and APLs and charges
#######################################################################

#You need to run a flat reference simualtions of the same
#system to get an approriate value for the membrane thickness
#and start values for the area per lipid (APL) for each lipid.
#APL and thickness can then be adjusted below.

#		lip1   lip2    apl1    apl2 charge1 charge2  ratio1  ratio2 thickness bulk_ions
full_run	POPC 	CDL0	0.68	0.94	0	0	0.9	0.1	3.98     0.15
