from numba import njit, prange
import numpy as np
import MDAnalysis as mda
import MDAnalysis.analysis.leaflet
from MDAnalysis.analysis import distances
from tqdm import tqdm
import sys
import re

np.set_printoptions(threshold=np.inf)
np.seterr(divide='ignore', invalid='ignore')

# load universe
path = sys.argv[1]
GRO = path + '/md.gro'
XTC = path + '/md.xtc'
u = mda.Universe(GRO, XTC)

# read data from reference wall (curvature and area per wall bead)
wall_setup = np.loadtxt('REF_WALL/Curvature_Wall.dat', comments="#", usecols=(4,6))
wall_ref = 'REF_WALL/wall.pdb'
wall = mda.Universe(wall_ref)
wall_pos = wall.select_atoms('resname WL').positions

# time to analyse
start_time = 3000		#in ns
end_time = 5000 		#in ns
dt = u.trajectory.dt/1000       #in ns

# lipids and their corresponding beads to be analyzed
#lipids = list(set(u.select_atoms("not resname W ION wall").resnames))
lipids = sys.argv[2:]
beads = []
for lipid in lipids:
    if re.search('CDL.', lipid): beads.append('GL0')
    elif re.search('CHOL', lipid): beads.append('ROH')
    elif re.search('.CE', lipid): beads.append('AM1')
    else: beads.append('PO4')

lipid_ratio = []
no_lipids = 0
for lipid, bead in zip(lipids, beads):
    count = len(u.select_atoms("resname {} and name {}".format(lipid, bead)))
    no_lipids += count
    lipid_ratio.append(count)

lipid_ratio = [val/no_lipids for val in lipid_ratio]
print(lipids)
print(['%.3f' % ratio for ratio in lipid_ratio ])
lipid_colors = [(0.17, 0.42, 0.54), (0, 1, 1) ]

def find_leaflets(u):

    # MDAnalysis leaflet finder works well for curved systems
    L = MDAnalysis.analysis.leaflet.LeafletFinder(u, 'name PO4 GL0 AM1', cutoff=20)

    #check leaflet detection
    if np.mean(L.group(0).positions, axis=0)[2] < np.mean(L.group(1).positions, axis=0)[2]:
        order = ['LL', 'UL']
    else: order = ['UL', 'LL']

    return(L, order)

def group_wall_beads(wall_pos_leaflet, offset):

    # wall beads are placed in rows with the same x and z postions (same curvature)
    # all data will be average over the wall beads that have the same x and z coordinates
    unique_wall_pos = []	# contains the first bead in a row of wall beads (with same curvature)
    wall_rows = []		# contains the number of beads in each row
    wall_curv = []		# contains the curvature of each unique wall bead
    wall_area = []              # contains the area of each unique wall bead
    for idx, pos in enumerate(wall_pos_leaflet):
        if idx == 0:
            unique_wall_pos.append(pos)
            wall_curv.append(wall_setup[offset+idx][0])
            wall_area.append(wall_setup[offset+idx][1])
            cnt = 1
        elif idx == len(wall_pos_leaflet)-1:
            cnt += 1
            wall_rows.append(cnt)
        else:
            if pos[0] != prev_pos[0]: #and pos[2] != prev_pos[2]:
                unique_wall_pos.append(pos)
                wall_curv.append(wall_setup[offset+idx][0])
                wall_area.append(wall_setup[offset+idx][1])
                wall_rows.append(cnt)
                cnt = 1
            else: cnt += 1
        prev_pos = pos
    return(np.array(unique_wall_pos), np.array(wall_rows), np.array(wall_curv), np.array(wall_area))

@njit(parallel=True)
def solv_dist_matrix(dist_arr, no_walls, no_lipids):

    sum_wall_lipids = np.zeros((no_walls))
    lipid_at_wall = np.zeros((no_lipids))

    for lipid_idx in prange(no_lipids):
        wall_idx = np.argmin(dist_arr[lipid_idx])
        #we consider only the first wall bead closest to a lipid bead
        lipid_at_wall[lipid_idx] = int(wall_idx)
        sum_wall_lipids[wall_idx] += 1

    return(sum_wall_lipids, lipid_at_wall)

def find_closest_wall_bead(lipid, selection, wall_pos_leaflet):

    # we count the number of lipid beads around each wall bead in each frame
    no_frames = int((end_time-start_time) / dt + 1 )
    lipids_around_wall_beads = np.zeros((no_frames, len(wall_pos_leaflet)))
    lipids_at_wall_beads = np.zeros((no_frames, len(selection)))
    dummy = np.zeros((len(selection),len(wall_pos_leaflet)))

    print('Finding the closest wall bead for each {} lipid'.format(lipid))
    for ts in tqdm(u.trajectory[int(start_time/dt):int(end_time/dt+1)]):
        frame = ts.frame - int(start_time/dt)
        #distance between all wall and lipid beads
        dist_arr = distances.distance_array(selection.positions, # reference
                                    wall_pos_leaflet, # configuration
                                    box=u.dimensions, result=dummy, backend="OpenMP")
        lipids_around_wall_beads[frame], lipids_at_wall_beads[frame] = solv_dist_matrix(dist_arr, len(wall_pos_leaflet), len(selection))
    box_size = ts.dimensions

    return(lipids_around_wall_beads, lipids_at_wall_beads, box_size)


def lipid_dist(lipid_count, unique_wall_pos, wall_rows, wall_curv, wall_area, leaflet):

    # Two-step averering of lipid_count:
    # 1: average over beads with same curvature (same x and z position)
    # 2: average number of lipids around each wall bead over time
    def average_data(data,  wall_rows, wall_area):

        # average over beads with same curvature (same x and z position)
        avg_row = np.zeros((len(data), len(wall_rows)))
        for frame, count in enumerate(data):
            start_idx = 0
            for idx, group in enumerate(wall_rows):
                end_idx = start_idx + group
                avg_row[frame][idx] = np.mean(count[start_idx:end_idx])/wall_area[idx]
                start_idx = end_idx

        # average number of lipids around each wall bead over time
        avg_lipids_around_wall_beads = np.mean(avg_row, axis=0)
        std_lipids_around_wall_beads = np.std(avg_row, axis=0, ddof=1) / np.sqrt(len(avg_row))

        return(avg_lipids_around_wall_beads, std_lipids_around_wall_beads)

    avg_lipids_around_wall_beads = []
    std_lipids_around_wall_beads = []

    for data in lipid_count:
        # get the average number of lipids per nm^2 over time and row
        avg, std = average_data(data, wall_rows, wall_area)
        avg_lipids_around_wall_beads.append(avg)
        std_lipids_around_wall_beads.append(std)

    return(np.array(avg_lipids_around_wall_beads), np.array(std_lipids_around_wall_beads))

@njit(parallel=True)
def average_data(data, wall_rows, wall_area):

    # average over beads with same curvature (same x and z position)
    avg_row = np.zeros((len(data), len(wall_rows)))

    for frame in prange(len(data)):
        start_idx = 0
        for idx, group in enumerate(wall_rows):
            end_idx = start_idx + group
            avg_row[frame][idx] = np.mean(data[frame][start_idx:end_idx])/wall_area[idx]
            start_idx = end_idx

    return(avg_row)


def quantify_sorting(lipid_count, wall_rows, wall_curv, wall_area):

# Sorting Index = (ratio of lipid x in positive curvature - ratio of lipid x in negative curvature )/ 2
# ratio of lipid x in positive curvature =
# no. of lipid x in + / total no. of lipids in + / expected ratio of lipid x in +
# the expected value comes from the setup

    lipid_pos_curv = np.zeros(lipid_count.shape[:-1])
    lipid_neg_curv = np.zeros(lipid_count.shape[:-1])

    for i, data in enumerate(lipid_count):

        # get the average number of lipids per nm^2 over time and row
        avg_lipid_count_row = average_data(data, wall_rows, wall_area)

        for frame in range(len(avg_lipid_count_row)):
            for j, curv in enumerate(wall_curv):

                if curv > 0: lipid_pos_curv[i][frame] += avg_lipid_count_row[frame][j]
                elif curv < 0: lipid_neg_curv[i][frame] += avg_lipid_count_row[frame][j]

    ratio_expect = np.array(lipid_ratio)
    de_pos = np.zeros(lipid_pos_curv.shape)
    de_neg = np.zeros(lipid_neg_curv.shape)
    sorting_idx = np.zeros(lipid_neg_curv.shape)

    ratio_pos = lipid_pos_curv / np.sum(lipid_pos_curv, axis=0)
    ratio_neg = lipid_neg_curv / np.sum(lipid_neg_curv, axis=0)

    for idx, (data_pos, data_neg) in enumerate(zip(ratio_pos, ratio_neg)):
        for frame in range(len(data)):
            de_pos[idx][frame] = data_pos[frame] / ratio_expect[idx]
            de_neg[idx][frame] = data_neg[frame] / ratio_expect[idx]
            sorting_idx[idx][frame] = (de_pos[idx][frame] - de_neg[idx][frame]) / 2
        print("Sorting index of {}: {:.3f} +/- {:.3f} (STD)".format(lipids[idx], np.mean(sorting_idx[idx]), np.std(sorting_idx[idx])))

    return(sorting_idx)


###########################################################################################################

leaflets, sorting = find_leaflets(u)
print('Number of lipids in the lower leaflet (LL) and upper leaflet (UL): ', len(leaflets.group(0)), len(leaflets.group(1)))

time_us = [(time*dt+start_time)/1000 for time in range(int((end_time-start_time)/dt+1))]

count_both_leaflets = []
curvature_both_leaflets = []
area_both_leaflets = []
lipid_dens_both_leaflets = []
dens_err_both_leaflets = []

for l, leaflet in enumerate(sorting):

    no_bottom_wall = no_top_wall = len(wall.atoms)//2
    if leaflet == 'LL':
        wall_pos_leaflet = wall_pos[:no_bottom_wall]
        offset = 0
    else:
        wall_pos_leaflet = wall_pos[no_bottom_wall:]
        offset = len(wall_pos)//2       #needed to read the coresponding curvatures and areas from wall_setup

    unique_wall_pos, wall_rows, wall_curv, wall_area = group_wall_beads(wall_pos_leaflet, offset)
    lipid_count = []

    lipid_dens = []
    dens_err = []

    print("\nAnalyze the {}\n".format(leaflet))
    for lcount, (lipid, bead) in enumerate(zip(lipids, beads)):

        resids = leaflets.group(l).resids
        selection = leaflets.group(l).select_atoms('resname {}'.format(lipid))
        print('Number of {} lipids in {} : {}'.format(lipid, leaflet, len(selection)))

        lipids_around_wall_beads, lipids_at_wall_beads, box_size = find_closest_wall_bead(lipid, selection, wall_pos_leaflet)
        lipid_count.append(lipids_around_wall_beads)

    count_both_leaflets.append(lipid_count)
    curvature_both_leaflets.append(wall_curv)
    area_both_leaflets.append(wall_area*wall_rows[0])

    lipid_dens, dens_err = lipid_dist(np.array(lipid_count), unique_wall_pos, wall_rows, wall_curv, wall_area, leaflet)
    sorting_idx = quantify_sorting(np.array(lipid_count), wall_rows, wall_curv, wall_area)

    #saving things to plot them later
    np.savetxt("sorting_index_{:1.2f}_{:1.2f}_{}.dat".format(time_us[0], time_us[-1], leaflet),
	np.vstack((time_us, sorting_idx)).T, fmt = '%.6f', header='time [us] '+' '.join(lipids))
    np.savetxt("density_{:1.2f}_{:1.2f}_{}.dat".format(time_us[0], time_us[-1], leaflet),
        np.vstack((wall_curv, wall_area*wall_rows[0], lipid_dens, dens_err)).T, fmt = '%.6f', header='curv [nm^-1] area [nm^2] density {0} {1} error {0} {1}'.format(lipids[0], lipids[1]))
