import numpy as np
import MDAnalysis as mda
from scipy.ndimage import gaussian_filter
import matplotlib.pyplot as plt
import pandas as pd
import sys

# Matplotlib parameters
plt.rcParams['font.family'] = "sans-serif"
plt.rcParams.update({'font.size': 42})
params = {'mathtext.default': 'regular' }
plt.rcParams.update(params)
plt.rcParams['axes.linewidth'] = 2

def plot_2D_density(density_matrix, wall, lipid):

    #data range
    y_max = density_matrix[-1,0].round()
    x_max = density_matrix[0,-1].round()
    y_steps = len(density_matrix[1:])
    x_steps = len(density_matrix[0,1:])

    if lipid == 'all': no_beads = len(u.select_atoms('not resname W ION'))
    else: no_beads = len(u.select_atoms('resname '+lipid))

    df = pd.DataFrame(density_matrix[1:,1:]/no_beads*1000,
           index=np.linspace(0,y_max,y_steps).round(decimals=1), columns=np.linspace(0,x_max, x_steps).round(decimals=1))
    #smoothing for contours
    smooth = gaussian_filter(df, sigma=1)
    df_smooth = pd.DataFrame(smooth,
           index=np.linspace(0,y_max,y_steps).round(decimals=1), columns=np.linspace(0,x_max, x_steps).round(decimals=1))

    xticks = np.arange(0, x_max+1, 5, dtype=int)
    yticks = np.arange(0, y_max+1, 5, dtype=int)

    x = np.arange(0,x_steps)
    y = np.arange(0,y_steps)
    X, Y = np.meshgrid(x, y)

    fig, ax = plt.subplots(figsize=(15, 12))
    im = ax.imshow(df.mask(df==0), cmap='jet', vmin=0, vmax=2.5, extent=(0, x_steps, 0, y_steps), origin='lower')

    ax.contour(X, Y, df_smooth.mask(df==0), 5, colors='k', linewidths=1, vmin=0, vmax=2.5, extent=(0, x_steps, 0, y_steps), origin='lower')

    # plot formating
    ax.set_xticks(xticks*(x_steps/x_max))
    ax.set_xticklabels(xticks, rotation=0)
    ax.set_yticks(yticks*(y_steps/y_max))
    ax.set_yticklabels(yticks, rotation=0)
    ax.set_xlim(0,x_steps)
    ax.set_ylim(0,y_steps)
    ax.tick_params(length=25, width=2, which='major')
    plt.xlabel("x [nm]")
    plt.ylabel("z [nm]")
    #add position of wall beads
    plt.scatter(wall[0], wall[1], color='#9500FF', s=500, edgecolors='k')

    # colorbar
    cbar = fig.add_axes([0.127, -0.07, 0.774, 0.05])
    cbar.tick_params(labelsize=32, direction='in', length=25, width=2, which='major')
    fig.colorbar(im, cax=cbar, orientation='horizontal', label='Rescaled number density (1x$10^{-3}$ $nm^{-3}$)')
    plt.savefig("avg_2d_dens_{}.png".format(lipid), bbox_inches="tight", dpi=300, transparent=True)
    plt.close()

#densities should be calculated as an average over several replicas

all_dens = []
for path in sys.argv[1:]:

    u = mda.Universe(path + '/md.gro')

    #check for lipid types
    lipids = ['all'] + list(set(u.select_atoms("not resname W ION wall").resnames))

    DensityFile = "-dens.dat"
    DensityFileWall = "Wall-dens.dat"

    # look for position of wall beads
    density_wall = np.loadtxt(DensityFileWall)
    wall = []
    for x in range(len(density_wall[1:])):
        for y in range(len(density_wall[0,1:])):
            if density_wall[1+x][1+y] > 0 and density_wall[1+x][y] == 0:
                wall.append([x,y])
    wall.append([len(density_wall[1:]),wall[0][1]])
    wall.append([len(density_wall[1:]),wall[1][1]])

    for idx, lipid in enumerate(lipids):
        density_matrix = np.loadtxt(lipid + DensityFile)
        all_dens.append(density_matrix)

#average over all provided folders
all_dens = np.reshape(all_dens, (len(sys.argv[1:]), len(lipids), density_matrix.shape[0], density_matrix.shape[1]))
avg_dens = np.mean(all_dens, axis=0)

#one plot per lipid type
for density, lipid in zip(avg_dens, lipids): plot_2D_density(density.T, np.array(wall).T, lipid)
