import numpy as np
from skmisc.loess import loess
import MDAnalysis as mda
import MDAnalysis.analysis.leaflet
import matplotlib.pyplot as plt
import sys
import re
from glob import glob
np.set_printoptions(threshold=np.inf)
np.seterr(divide='ignore', invalid='ignore')

# Matplotlib parameters
plt.rcParams['font.family'] = "sans-serif"
plt.rcParams.update({'font.size': 42})
params = {'mathtext.default': 'regular' }
plt.rcParams.update(params)
plt.rcParams['axes.linewidth'] = 2

# load universe
path = sys.argv[1]
GRO = path + '/md.gro'
u = mda.Universe(GRO)

lipids = sys.argv[2:]

#========================================================================================================

def find_leaflets(u):

    # MDAnalysis leaflet finder works well for curved systems
    L = MDAnalysis.analysis.leaflet.LeafletFinder(u, 'name PO4 GL0 AM1', cutoff=20)

    #check leaflet detection
    if np.mean(L.group(0).positions, axis=0)[2] < np.mean(L.group(1).positions, axis=0)[2]:
        order = ['LL', 'UL']
    else: order = ['UL', 'LL']

    return(L, order)

def lowess_curve(x, y):

    z = np.linspace(-1,np.max(x),500)
    l = loess(x, y, span=0.2, surface='direct')
    l.fit()
    pred = l.predict(z, stderror=True)
    lowess = pred.values
    conf = pred.confidence()
    ll = conf.lower
    ul = conf.upper

    return(z, lowess, ll, ul)

def plot_relative_dens(total_dens, total_expected_dens, curv):

    fig, ax = plt.subplots(figsize=(15, 12))

    all_curv = curv.flatten()
    all_relative_dens = (total_dens/total_expected_dens[:,None]).flatten()

    #local polynomial regression
    z, lowess, ll, ul = lowess_curve(all_curv, all_relative_dens)
    ax.plot(z, lowess, lw=3, color="limegreen")
    ax.fill_between(z, ll, ul, alpha=.2, lw=0.0, color="limegreen")

    ax.axhline(y=1, color='black', linestyle=(0,(5, 5)), linewidth=2)
    ax.set_xlim(-0.41,0.11)
    ax.set_ylim(0.7, 1.4)
    ax.tick_params(length=25, width=2, which='major')
    ax.set_xlabel(r"Mean curvature [nm$^{\rm -1}$]")
    ax.set_ylabel(r"Relative density")

    # save figure
    plt.savefig("Relative_density.png", bbox_inches="tight", dpi=300)
    plt.close()

def plot_enrichment(dens, expected_dens, total_dens, total_expected_dens, curv):

    fig, ax = plt.subplots(figsize=(15, 12))

    all_curv = curv.flatten()
    all_relative_dens = total_dens/total_expected_dens[:,None]

    for idx, lipid in enumerate(lipids):

        relative_dens = dens[:,idx]/expected_dens[:,idx][:,None]
        relative_enrichment = relative_dens/all_relative_dens

        #local polynomial regression
        z, lowess, ll, ul = lowess_curve(all_curv, relative_enrichment.flatten())
        ax.plot(z, lowess, lw=3, label=lipid)
        ax.fill_between(z, ll, ul, alpha=.2, lw=0.0)

    ax.axhline(y=1, color='black', linestyle=(0,(5, 5)), linewidth=2)
    ax.set_xlim(-0.41,0.11)
    ax.set_ylim(0, 3)
    ax.tick_params(length=25, width=2, which='major')
    ax.legend(frameon=False)
    ax.set_xlabel(r"Mean curvature [nm$^{\rm -1}$]")
    ax.set_ylabel(r"Relative enrichment")

    # save figure
    plt.savefig("Relative_enrichment.png", bbox_inches="tight", dpi=300)
    plt.close()

#========================================================================================================

dens = []
curv = []
area = []
expected_dens = []

leaflets, sorting = find_leaflets(u)

for l, leaflet in enumerate(sorting):

    #load density data
    filename = glob('density_*_{}.dat'.format(leaflet))
    print('Reading file ' + filename[0])
    data = np.loadtxt(filename[0], comments='#')
    curv.append(data.T[0])
    area.append(data.T[1])
    dens.append(data.T[2:2+len(lipids)])

    for lipid in lipids:

        sel = leaflets.group(l).select_atoms('resname {}'.format(lipid))
        expected_dens.append(len(sel)/np.sum(area[l]))

expected_dens = np.reshape(expected_dens, (len(lipids),len(lipids)))
total_dens = np.sum(dens, axis=0)
total_expected_dens = np.sum(expected_dens, axis=0)

#This plots the relative lipids density:
#Relative density is defined as the density of all lipids divided by the
#expected denisty,e.g., a value larger than 1 means that the packing of
#lipids is tighter compared to a flat membrane.
plot_relative_dens(total_dens, total_expected_dens, np.array(curv))

#This plots the actual lipid enrichment:
#Lipid enrichment is defined as the relative lipid density of each
#lipid type dived by the total relative density, e.g., a value larger
#than 1 means that the lipid type is enriched.
plot_enrichment(np.array(dens), np.array(expected_dens), total_dens, total_expected_dens, np.array(curv))
